*******************************************************************
* Calculation of poverty PPPs
*
* Program   : 01d_ppp_nipa_105_ALL.do
* Authors   : Angus Deaton, Princeton University
*           : Olivier Dupriez, World Bank
* Version   : April 6, 2009
* Purpose   : Calculates cpd, fisher and tornqvist PPP index using 
*             consumption weights from national accounts, for all possible
*             countries (and not only for the 62 for which we have P4s)
*
* This Stata program is part of the following collection of Stata [.do] files:

* 
* Preparation of data files:
*   00_data_reshape_102_105
*   00_data_reshape_102_105_ALL
*
* Calculation of PPPs (NIPA and survey weights):
*   01a_ppp_nipa_102
*   01b_ppp_nipa_105
*   01c_ppp_nipa_102_ALL
*   01d_ppp_nipa_105_ALL
*   02_cpd_index_surveys
*   03_super_index_surveys
*
* Calculation of PPPPs and US$ index- Option "14 unweighted poverty lines":
*   104_tornqvist_approximation_14plnw
*   105_poverty_indexes_cpd_14plnw
*   106_poverty_indexes_cpd_sterrs_14plnw
*   107_poverty_indexes_fisher_14plnw
*   108_poverty_indexes_fisher_sterrs_14plnw
*   109_poverty_indexes_tornqvist_14plnw
*   110_poverty_indexes_tornqvist_sterrs_14plnw
*   111_cpd_USD_14plnw
*   112_fisher_USD_14plnw
*   113_tornqvist_USD_14plnw
*
* Calculation of PPPPs and US$ index- Option "50 weighted poverty lines":
*   204_tornqvist_approximation_50plw
*   205_poverty_indexes_cpd_50plw
*   206_poverty_indexes_cpd_sterrs_50plw
*   207_poverty_indexes_fisher_50plw
*   208_poverty_indexes_fisher_sterrs_50plw
*   209_poverty_indexes_tornqvist_50plw
*   210_poverty_indexes_tornqvist_sterrs_50plw
*   211_cpd_USD_50plw
*   212_fisher_USD_50plw
*   213_tornqvist_USD_50plw
*
* Calculation of PPPPs and US$ index- Option "50 weighted and doubled poverty lines":
*   304_tornqvist_approximation_50plw2
*   305_poverty_indexes_cpd_50plw2
*   306_poverty_indexes_cpd_sterrs_50plw2
*   307_poverty_indexes_fisher_50plw2
*   308_poverty_indexes_fisher_sterrs_50plw2
*   309_poverty_indexes_tornqvist_50plw2
*   310_poverty_indexes_tornqvist_sterrs_50plw2
*   311_cpd_USD_50plw2
*   312_fisher_USD_50plw2
*   313_tornqvist_USD_50plw2
*
* Star system for conversion to US$ for estimates from national accounts:
*   411_cpd_USD_NIPA102
*   412_fisher_USD_NIPA102
*   413_tornqvist_USD_NIPA102
*   511_cpd_USD_NIPA105
*   512_fisher_USD_NIPA105
*   513_tornqvist_USD_NIPA105
*
* Estimation of PPPs by regression model for non-ICP or non-survey countries:
*   601_regressions_PPP_estimates
*
**************************************************************************** 

* We will calculate PPPs for the following countries:

* India		in
* Brunei	bn
* Cambodia	kh
* China		cn
* Hong Kong	hk
* Macao		mo
* Taiwan	tw
* Indonesia	id
* Lao PDR	la
* Malaysia	my
* Mongolia	mn
* Philippines	ph
* Singapore	sg
* Thailand	th
* Vietnam	vn
* Bangladesh	bd
* Bhutan	bt
* Maldives	mv
* Nepal		np
* Pakistan	pk
* Sri Lanka	lk
* Fiji		fj
* Argentina Urb.ar
* Bolivia	bo
* Brazil	br
* Chile		cl
* Colombia	co
* Ecuador	ec
* Paraguay	py
* Peru		pe
* Uruguay	uy
* Venezuela, RB	ve
* Albania	al
* Armenia	am
* Azerbaijan	az
* Croatia	hr
* Czech Rep.	cz
* Estonia	ee
* Georgia	ge
* Hungary	hu
* Kazakhstan	kz
* Kyrgyz Rep.   kg
* Latvia	lv
* Lithuania	lt
* Macedonia,FYR mk
* Poland	pl
* Romania	ro
* Russian Fed.	ru
* Slovak Rep.	sk
* Slovenia	si
* Tajikistan	tj
* Turkey	tr
* Ukraine	ua
* Angola	ao
* Benin		bj
* Botswana	bw
* Burkina Faso	bf
* Burundi	bi
* Cameroon	cm
* Cape Verde	cv
* Cent.Afr. Rep.cf
* Chad		td
* Comoros	km
* Congo, Dem. R.cd
* Congo, Rep.	cg
* Cte d'Ivoire	ci
* Equat.Guinea	gq
* Ethiopia	et
* Gabon		ga
* Gambia, The	gm
* Ghana		gh
* Guinea	gn
* Guinea Bissau	gw
* Kenya		ke
* Lesotho	ls
* Liberia	lr
* Madagascar	mg
* Malawi	mw
* Mali		ml
* Mauritania	mr
* Mauritius	mu
* Mozambique	mz
* Namibia	na
* Niger		ne
* Nigeria	ng
* Rwanda	rw
* Sao Tome Pr.	st
* Senegal	sn
* Sierra Leone	sl
* South Africa	za
* Sudan		sd
* Swaziland	sz
* Tanzania	tz
* Togo		tg
* Uganda	ug
* Zambia	zm
* Zimbabwe	zw
* Djibouti	dj
* Egypt, Arab R.eg
* Iran, Isl.Rep.ir
* Iraq		iq
* Jordan	jo
* Kuwait	kw
* Lebanon	lb
* Morocco	ma
* Oman		om
* Qatar		qa
* Saudi Arabia	sa
* Syrian Arab R.sy
* Tunisia	tn
* Yemen, Rep.	ye

* We will have to estimate them by regression for the following:

*Papua New Guin.pg
*Timor Leste	tl
*Costa Rica	cr
*Dominican Rep.	do
*El Salvador	sv
*Guatemala	gt
*Guyana		gy
*Haiti		ht
*Honduras	hn
*Jamaica	jm
*Mexico		mx
*Nicaragua	ni
*Panama		pa
*St. Lucia	lc
*Suriname	sr
*Trinidad Tob.	tt
*Belarus	by
*Bosnia Herz.	ba
*Bulgaria	bg
*Moldova, Rep.	md
*Turkmenistan	tm
*Uzbekistan	uz
*Algeria	dz
* 
****************************************************************************

version 9.2
#delimit;
set logtype text;
clear;
set matsize 8000;
set mem 80m;
set more off;
cap log close; 
clear;
drop _all;

* >> Enter directories where datafiles are found, and calculations have to be saved;
local dirdata  = "D:\_P4_Final\P4_Input_Files";
local dircalc  = "D:\_P4_Final\P4_Calculations";

cd "`dirdata'";

* >> Enter number of basic headings to be used in calculations;

global ng=105;

* >> Enter list of countries to be included in the calculation;
global cns1= "in bn kh cn hk mo tw id la my mn ph sg th vn bd bt mv np pk lk fj ar bo br cl co ec mx py pe 
              uy ve al am az by ba bg hr cz ee ge hu kz kg lv lt mk md pl ro ru sk si tj tr ua";
global cns2= "ao bj bw bf bi cm cv cf td km cd cg ci gq et ga gm gh gn gw ke ls lr mg mw ml mr mu mz na 
              ne ng rw st sn sl za sd sz tz tg ug zm zw dj eg ir iq jo kw lb ma om qa sa sy tn ye"; 

*We count the number of countries;
global nc=0;
foreach iso of global cns1 {;
    global nc=$nc+1;
};
foreach iso of global cns2 {;
    global nc=$nc+1;
};
display "Number of countries is: " $nc;

* >> Enter name and location of input and log files;
local icprice  = "`dirdata'\bh_icp_prices_"+"$ng"+"_Final_without_OECD_ALL.dta";
local logf     = "`dircalc'\PPP_NIPA_"+"$ng"+"_$nc_ALL.smcl";
log using "`logf'", replace;

* We create a file with consumption shares and parities at basic heading level;
use `icprice';
sort basic_hd;
keep basic_hd p_*;
merge basic_hd using "new_bh_nat_accounts_$ng.dta";
tab _m;
keep if _m==3;
drop _m;
assert _N==$ng;

* There can be no missing data in parities or consumption shares;
* If some prices were missing, they would have to be estimated;

* Replace missing consumption shares with 0;
foreach co of global cns1 {;
   recode x_`co'(.=0);
};
foreach co of global cns2 {;
   recode x_`co'(.=0);
};

* >> Select base country;
* Make India the base country;
mkmat p_in;
mkmat x_in;
matrix p=p_in;
matrix x=x_in;

foreach co of global cns1 {;
   mkmat p_`co';
   mkmat x_`co';
   if "`co'" ~="in" {;
      matrix p=p,p_`co';
      matrix x=x,x_`co';
   };
};
foreach co of global cns2 {;
   mkmat p_`co';
   mkmat x_`co';
   if "`co'" ~="in" {;
      matrix p=p,p_`co';
      matrix x=x,x_`co';
   };
};

matrix aye=J($ng,1,1);
matrix xtot=x'*aye;
matrix w=x*invsym(diag(xtot));

matrix w_1=w[1..$ng,1];
matrix star_ppp_l=p'*w_1;

**Laspeyres star PPPs;
matrix list star_ppp_l;

matrix r_1=J($ng,$nc,1);
matrix lnp=J($ng,$nc,1);

forvalues i=1(1)$ng {;
   forvalues j=1(1)$nc {;
      matrix r_1[`i',`j']=1/p[`i',`j'];
      matrix lnp[`i',`j']=log(p[`i',`j']);
   };
};

matrix tm=r_1'*w;
matrix star_ppp_p=J($nc,1,1);

forvalues i=1(1)$nc {;
   matrix star_ppp_p[`i',1]=1/tm[`i',`i'];
};

**Paasche star PPPs;
matrix list star_ppp_p;

matrix star_ppp_f=J($nc,1,1);

forvalues i=1(1)$nc {;
   matrix star_ppp_f[`i',1]=sqrt(star_ppp_p[`i',1]*star_ppp_l[`i',1]);
};

**Fisher star PPPs;
matrix list star_ppp_f;

**Now doing the CPD PPPs;

drop _all;
svmat p;
svmat w;
gen gd=_n;

reshape long p w, i(gd) j(iso_no);
gen lnp=log(p);

xi: regress lnp i.gd i.iso_no [aweight=w];

matrix b=e(b);

matrix ppp_cpd=J($nc,1,1);

forvalues i=2(1)$nc {;
   matrix aa=b[1,"_Iiso_no_`i'"];
   matrix ppp_cpd[`i',1]=exp(aa[1,1]);
};

***CPD PPPs;
matrix list ppp_cpd;

**now doing the EKS versions of Fisher and Tornqvist;

matrix plasp=J($nc,$nc,1);
matrix pfish=J($nc,$nc,1);
matrix plogfish=J($nc,$nc,1);
matrix ptorn=J($nc,$nc,1);

forvalues i=1(1)$nc {;
   forvalues j=1(1)$nc {;
      matrix at=invsym(diag(p[1..$ng,`i']))*p[1..$ng,`j'];
      matrix plasp[`i',`j']=w[1..$ng,`i']'*at;
   };
};

forvalues i=1(1)$nc {;
   forvalues j=1(1)$nc {;
      scalar at=plasp[`i',`j']/plasp[`j',`i'];
      matrix pfish[`i',`j']=sqrt(at);
      matrix plogfish[`i',`j']=log(sqrt(at));
      matrix ptorn[`i',`j']=0.5*(w[1..$ng,`i']+w[1..$ng,`j'])'*(lnp[1..$ng,`j']-lnp[1..$ng,`i']);
   };
};

matrix eks_fish=((plogfish'+J($nc,1,1)*plogfish[1,1..$nc]))*J($nc,1,1);
matrix eks_fish=eks_fish/$nc;
matrix eks_torn=((ptorn'+J($nc,1,1)*ptorn[1,1..$nc]))*J($nc,1,1);
matrix eks_torn=eks_torn/$nc;

forvalues i=1(1)$nc {;
   matrix eks_fish[`i',1]=exp(eks_fish[`i',1]);
   matrix eks_torn[`i',1]=exp(eks_torn[`i',1]);
};

matrix list eks_fish;
matrix list eks_torn;

matrix ppp=ppp_cpd,eks_fish,eks_torn;

display("All three PPPs: CPD, EKS-Fisher, EKS-Tornqvist");
matrix list ppp;

log close;