*******************************************************************
* Calculation of poverty PPPs
*
* Program   : 204_tornqvist_approximation.do
* Authors   : Angus Deaton, Princeton University
*           : Olivier Dupriez, World Bank
* Version   : December 25, 2008
* Purpose   : Makes the Tornvist approximation to the PPPP  
*             (m-country case), with option "50 weighted poverty lines" 
* Note      : This program requires the prices and weight file 
*             produced by program 02_cpd_index_surveys.do   
*
* This Stata program is part of the following collection of Stata [.do] files:

* 
* Preparation of data files:
*   00_data_reshape_102_105
*   00_data_reshape_102_105_ALL
*
* Calculation of PPPs (NIPA and survey weights):
*   01a_ppp_nipa_102
*   01b_ppp_nipa_105
*   01c_ppp_nipa_102_ALL
*   01d_ppp_nipa_105_ALL
*   02_cpd_index_surveys
*   03_super_index_surveys
*
* Calculation of PPPPs and US$ index- Option "14 unweighted poverty lines":
*   104_tornqvist_approximation_14plnw
*   105_poverty_indexes_cpd_14plnw
*   106_poverty_indexes_cpd_sterrs_14plnw
*   107_poverty_indexes_fisher_14plnw
*   108_poverty_indexes_fisher_sterrs_14plnw
*   109_poverty_indexes_tornqvist_14plnw
*   110_poverty_indexes_tornqvist_sterrs_14plnw
*   111_cpd_USD_14plnw
*   112_fisher_USD_14plnw
*   113_tornqvist_USD_14plnw
*
* Calculation of PPPPs and US$ index- Option "50 weighted poverty lines":
*   204_tornqvist_approximation_50plw
*   205_poverty_indexes_cpd_50plw
*   206_poverty_indexes_cpd_sterrs_50plw
*   207_poverty_indexes_fisher_50plw
*   208_poverty_indexes_fisher_sterrs_50plw
*   209_poverty_indexes_tornqvist_50plw
*   210_poverty_indexes_tornqvist_sterrs_50plw
*   211_cpd_USD_50plw
*   212_fisher_USD_50plw
*   213_tornqvist_USD_50plw
*
* Calculation of PPPPs and US$ index- Option "50 weighted and doubled poverty lines":
*   304_tornqvist_approximation_50plw2
*   305_poverty_indexes_cpd_50plw2
*   306_poverty_indexes_cpd_sterrs_50plw2
*   307_poverty_indexes_fisher_50plw2
*   308_poverty_indexes_fisher_sterrs_50plw2
*   309_poverty_indexes_tornqvist_50plw2
*   310_poverty_indexes_tornqvist_sterrs_50plw2
*   311_cpd_USD_50plw2
*   312_fisher_USD_50plw2
*   313_tornqvist_USD_50plw2
*
* Star system for conversion to US$ for estimates from national accounts:
*   411_cpd_USD_NIPA102
*   412_fisher_USD_NIPA102
*   413_tornqvist_USD_NIPA102
*   511_cpd_USD_NIPA105
*   512_fisher_USD_NIPA105
*   513_tornqvist_USD_NIPA105
*
* Estimation of PPPs by regression model for non-ICP or non-survey countries:
*   601_regressions_PPP_estimates
*
****************************************************************************
*
* Parameters (see " * >> " in the code below):
*    Number of basic headings
*    Location (folders) of input/output files 
*    List of countries (iso code)
*    Reference poverty line used for approximation
*
****************************************************************************

version 9.2
#delimit;
set logtype text;
clear;
set mem 100m;
set matsize 6350;
set more off;
pause on;
cap log close; 
clear;

* The program requires information on the mean poverty line to be used.   ;
* This poverty line varies according to the number of national poverty    ;
* lines that need to be included in the calculation of the international  ;
* poverty line. In our calculations, we use 14 npls, 50npls, and          ; 
* 50npls multiplied by 2.                                                 ;

* >> Select the option here (either 14plnw, 50plw, or 50plw2)             ;
local ploption="50plw";

* >> Enter directories where datafiles are found, and calculations have to be saved;
local dirdata  = "C:\Users\Olivier\_P4_Final\P4_Input_Files";
local dircalc  = "C:\Users\Olivier\_P4_Final\P4_Calculations";
cd "`dirdata'";

* >> Enter number of basic headings to be used in calculations;
local ng=102;

* >> Enter the list of countries to include in calculations;
global cns= "in bd bt kh cn fj id la my mv mn np pk ph lk th vn bj bf bi cm cv td cd cg ci dj et ga gm 
gh gn ke ls mg mw ml mr ma mz ne ng rw sn sl za sz tz tg ug ar bo br co py pe am az kz kg tj ye";

*We count the number of countries;
global nc=0;
foreach iso of global cns {;
    global nc=$nc+1;
};
display "Number of countries is: " $nc;

* >> Enter location of input and log files to be used;
local prices_w = "`dirdata'\Global_prices_w_`ng'_Final.dta";
log using "`dircalc'\Tornqvist_approx_50plw_`ng'_$nc.smcl", replace;

* We check that all necessary files exist in the specified location;
confirm file "`dirdata'\Global_prices_w_`ng'_Final.dta";
foreach iso of global cns {;
    confirm file "expsurv`ng'_`iso'.dta";
};

* >> Enter the poverty line used as reference (pl0) ;

* We use a weighted average of 50 national poverty lines.          ;
* These PLs are all expressed in 2005 prices, and converted        ;
* into Rupees using the Tornqvist calculated with 03_super_index.  ;
* See calculation of that PL in file pov_lines_for_PPPP.xls        ;
 
if "`ploption'"=="14plnw" {;
   scalar pl0=6104.203; 
};
if "`ploption'"=="50plw" {;
   scalar pl0=7714.923; 
};
if "`ploption'"=="50plw2" {;
   scalar pl0=15429.847; 
};

scalar lnz=log(pl0);

forvalues i=1(1)$nc {;
   matrix xei`i'=J(`ng',2,0);
};

global cno=1;

foreach iso of global cns {;
   forvalues i=1(1)`ng' {;
      drop _all;
      use x`i' sx mult hhs hh_totx using expsurv`ng'_`iso';
      drop if hh_totx==. | hh_totx==. | sx==. | inlist(hhs,.,0);
      gen lnpce=log(hh_totx/hhs);
      gen w=x`i'/sx;
      qui regress w lnpce [aweight=mult];
      matrix xei$cno[`i',1]=_b[_cons];
      matrix xei$cno[`i',2]=_b[lnpce]; 
   };
   global cno=$cno+1;
};

drop _all;
use "`prices_w'";
global cno=1;

foreach iso of global cns {;
   ren p_`iso' p$cno;
   gen lnp$cno=log(p$cno);
   mkmat lnp$cno;
   global cno=$cno+1;
};

drop _all;

matrix zeta=J($nc,$nc,0);
matrix phi=J($nc,$nc,0);
matrix psi=J($nc,$nc,0);

forvalues i=1(1)$nc {;
   forvalues j=1(1)$nc {;
      matrix zeta[`i',`j']=0.5*(xei`i'[1..`ng',1]+xei`j'[1..`ng',1]+xei`i'[1..`ng',2]*lnz+xei`j'[1..`ng',2]*lnz)'*(lnp`j'-lnp`i');
      matrix phi[`i',`j']=0.5*xei`i'[1..`ng',2]'*(lnp`j'-lnp`i');
      matrix psi[`i',`j']=0.5*xei`j'[1..`ng',2]'*(lnp`j'-lnp`i');
   };
};

matrix uns=J($nc,1,1);
matrix a=(phi'+uns*psi[1,1..$nc])/$nc+(I($nc)-(1/$nc)*diag(psi'*uns));
matrix ax=J($nc,$nc,0);
matrix ax[1,1]=(phi[1,1..$nc]*uns)/$nc;
matrix a=a+ax;
matrix b=((zeta[1,1..$nc]*uns)*uns+zeta'*uns)/$nc;
matrix ans=inv(a)*b;

local cc=colsof(ans);
local rr=rowsof(ans);
matrix pppans=J(`rr',`cc',0);
forvalues i=1(1)`rr' {;
   forvalues j=1(1)`cc' {;
      matrix pppans[`i',`j']=exp(ans[`i',`j']);
   };
};

display("PPPP Tornqvist approximations");
matrix list ans;
matrix list pppans;

* We save the reference poverty line and the approximation of P4s in a file for use in next programs;
matrix approx=J($nc, 1,0);
matrix approx=approx,ans;
matrix approx[1,1]=pl0;
matrix approx[2,1]=$nc;
matrix approx[3,1]=`ng';
svmat approx;
save "`dirdata'\Torn_Approx_50plw_`ng'_$nc.dta", replace;

log close;